#!/bin/bash
#-----------------------------#
# setting                     #
#-----------------------------#

# mailto
mailto='root@hogehoge.com"
# mail subject
subject_str='s25rtarpitgreylist report'
# print record
num_record=20
# path of maillog
path_log_default='/var/log/maillog'
# path of tmpfile
path_tmp=/tmp/${0##*/}_tmp.$$
path_mailbody_tmp=/tmp/${0##*/}_mail.$$

#-----------------------------#
# proc                        #
#-----------------------------#

LANG=C

function delete_tmpfile {
	rm -f ${path_tmp}
	rm -f ${path_mailbody_tmp}
}

function send_mail {
	# ---- get arg ----
	for arg in "$@"
	do
		if [[ ${arg} = -? ]];then
			preflag=${arg}
		else
			# check flag
			if [[ ${preflag} = "-f" ]];then
				# When the path of the file which wrote the content of the email was set in an argument
				msgbodyfile="${arg}"
			elif [[ ${preflag} = "-m" ]];then
				# When the contents of the message are carried with an argument
				msgstr_arg="${arg}"
			elif [[ ${preflag} = "-s" ]];then
				# When it set a subject
				subject_str="${arg}"
			elif [[ ${preflag} = "-t" ]];then
				# When It set an address
				mail_to="${arg}"
			elif [[ ${preflag} = "-l" ]];then
				# message level
				level="${arg}"
			fi
			# clear flag
			preflag=""
	
		fi
	done
	
	# When the contents of the message are a file or arguments, and it is not cooperated
	if [[ -z "${msgbodyfile}" && -z "${msgstr_arg}" ]];then
		# get message from standart io
		msgstr=$( cat )
	elif [[ ! -z ${msgbodyfile} ]];then
		# When the contents of the message are carried with an argument
		# get message from file
		if [[ -r ${msgbodyfile} ]];then
			msgstr=$( cat ${msgbodyfile} )
		fi
	else
		# get message from arg
		msgstr="${msgstr_arg}"
	fi
	
	# ---- add subject ----
	if [[ ${level} = "I" ]];then
		subject_str="[INFO] ${subject_str}"
	elif [[ ${level} = "W" ]];then
		subject_str="[WARN] ${subject_str}"
	elif [[ ${level} = "E" ]];then
		subject_str="[ERR ] ${subject_str}"
	fi

	mail  -s "$(echo ${subject_str} | iconv -f UTF-8 -t ISO-2022-JP)" "${mailto}" << MAILEND
$(echo "${msgstr}" | iconv -f UTF-8 -t ISO-2022-JP )
MAILEND
}

if [[ $# = 1 ]];then
	path_log=$1
else
	path_log=${path_log_default}
fi
if [[ ! -f ${path_log} ]];then
	echo "Can't read maillog : ${path_log}"
	exit 1
fi

# Get yesterday
str_yesterday=$( date "+%b %e" --date '1 days ago')
str_yesterday_mail=$( date '+%Y-%m-%d' --date '1 days ago')
if [[ $# = 2 ]];then
	str_yesterday=$2
fi

# Get yesterday only log
grep "^${str_yesterday} " ${path_log} > ${path_tmp}
grep "^${str_yesterday} " ${path_log}.1 >> ${path_tmp}
#cat /var/log/maillog.1 >  ${path_tmp}

echo "===== SUMMARY ===================" > ${path_mailbody_tmp}
cnt_all=$( egrep " greylist\[.+\) INFO \: \- \: start" ${path_tmp} | wc -l | awk '{print $1}' )
printf "%d %s\n" ${cnt_all} "all" >> ${path_mailbody_tmp}

cnt_ng_all=$( egrep " greylist\[.+\) NG \:" ${path_tmp} | wc -l | awk '{print $1}' )
printf "%d %s\n" ${cnt_ng_all} "ng all" >> ${path_mailbody_tmp}

echo "===== NG reason ===================" >> ${path_mailbody_tmp}
egrep " greylist\[.+\) NG \:" ${path_tmp} | awk '{printf "%s %s\n",$8,$10}'| sort | uniq -c >> ${path_mailbody_tmp}

echo "===== NG host top${num_record} ===================" >> ${path_mailbody_tmp}
egrep " greylist\[.+\) NG \:" ${path_tmp} | awk '{printf "%s %s\n",$6,$7}' | sort | uniq -c | sort -r | head -n ${num_record} >> ${path_mailbody_tmp}

echo "===== NG host top${num_record}(reason) ===================" >> ${path_mailbody_tmp}
egrep " greylist\[.+\) NG \:" ${path_tmp} | tr -s ' ' | cut -d' ' -f 6- | sort | uniq -c | sort -r | head -n ${num_record} >> ${path_mailbody_tmp}

echo "===== NG host helocheck top${num_record}(reason) ===================" >> ${path_mailbody_tmp}
egrep " greylist\[.+\) NG :.+ different from HELO host in connection host" ${path_tmp} | tr -s ' ' | cut -d' ' -f 6- | sort | uniq -c | sort -r | head -n ${num_record} >> ${path_mailbody_tmp}

echo "===== NG host comesoon top${num_record} ===================" >> ${path_mailbody_tmp}
egrep " greylist\[.+\) NG :.+ The host who came more than" ${path_tmp} | awk '{printf "%s %s\n",$6,$7}' | sort | uniq -c | sort -r | head -n ${num_record} >> ${path_mailbody_tmp}

echo "===== 450 host top${num_record} ===================" >> ${path_mailbody_tmp}
egrep " greylist\[.+\) INFO :.+ connection closed \((quit|45\?)\)" ${path_tmp} | awk '{printf "%s %s\n",$6,$7}' | sort | uniq -c | sort -r | head -n ${num_record} >> ${path_mailbody_tmp}

echo "===== OK regist greylist top${num_record}  ===================" >> ${path_mailbody_tmp}
egrep " greylist\[.+\) OK :.+accepting$" ${path_tmp} | awk '{printf "%s %s\n",$6,$7}' | sort | uniq -c | sort -r | head -n ${num_record} >> ${path_mailbody_tmp}

echo "===== ERROR (reason) ===================" >> ${path_mailbody_tmp}
egrep " greylist\[.+\) ERR :" ${path_tmp} | tr -s ' ' | cut -d' ' -f 6- | sort | uniq -c | sort -r  >> ${path_mailbody_tmp}
echo "===== WARNING (reason) ===================" >> ${path_mailbody_tmp}
egrep " greylist\[.+\) WARN :" ${path_tmp} | tr -s ' ' | cut -d' ' -f 6- | sort | uniq -c | sort -r  >> ${path_mailbody_tmp}


send_mail -f ${path_mailbody_tmp} -s "${subject_str} (${str_yesterday_mail})" -t "${mailto}"

delete_tmpfile
